/*
 * file: pv_set.h
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#ifndef PV_SET_H
#define PV_SET_H

#include <libpv_config.h>

#include <inttypes.h>
#include <errno.h>

#include <pv_types.h>
#include <pv_register.h>

#ifndef CONFIG_IGNORE_LIMITS
#define PV_SET_VALUE(pv, type, val) \
	if (((pv)->has_min && ((val) < (pv)->min.type)) || \
	    ((pv)->has_max && ((val) > (pv)->max.type))) \
		return ERANGE; \
	*(pv_to_abs_addr(engine, pv)->type) = val; \
	return 0;
#else
#define PV_SET_VALUE(pv, type, val) \
	*(pv_to_abs_addr(engine, pv)->type) = val; \
	return 0;
#endif


static inline int pv_set_uint8	(const struct pv_engine	*engine,
				 const struct pv	*pv,
				 uint8_t		val)
{
	PV_SET_VALUE(pv, u8, val);
}

static inline int pv_set_int8	(const struct pv_engine	*engine,
				 const struct pv	*pv,
				 int8_t			val)
{
	PV_SET_VALUE(pv, s8, val);
}


static inline int pv_set_uint16(const struct pv_engine	*engine,
				 const struct pv	*pv,
				 uint16_t		val)
{
	PV_SET_VALUE(pv, u16, val);
}

static inline int pv_set_int16	(const struct pv_engine	*engine,
				 const struct pv	*pv,
				 int16_t		val)
{
	PV_SET_VALUE(pv, s16, val);
}


static inline int pv_set_uint32(const struct pv_engine	*engine,
				 const struct pv	*pv,
				 uint32_t		val)
{
	PV_SET_VALUE(pv, u32, val);
}

static inline int pv_set_int32	(const struct pv_engine	*engine,
				 const struct pv	*pv,
				 int32_t		val)
{
	PV_SET_VALUE(pv, s32, val);
}


static inline int pv_set_uint64(const struct pv_engine	*engine,
				 const struct pv	*pv,
				 uint64_t		val)
{
	PV_SET_VALUE(pv, u64, val);
}

static inline int pv_set_int64	(const struct pv_engine	*engine,
				 const struct pv	*pv,
				 int64_t		val)
{
	PV_SET_VALUE(pv, s64, val);
}

static inline int pv_set_float	(const struct pv_engine	*engine,
				 const struct pv	*pv,
				 float			val)
{
	PV_SET_VALUE(pv, f, val);
}

static inline int pv_set_double	(const struct pv_engine	*engine,
				 const struct pv	*pv,
				 double			val)
{
	PV_SET_VALUE(pv, d, val);
}

#endif	/* !PV_SET_H */
